# SPDX-License-Identifier: MIT


# Import local modules.
from .main import *

# Set package information.
__description__ = 'A modern Pythonic implementation of Avro Phonetic.'

__version_info__ = ('2024', '4', '7')  # YYYY / MM / DD
__version__ = '.'.join(__version_info__)
